//
// Copyright (c) 2012-2016 Michele Segata <segata@ccs-labs.org>, Tobias Hardes <hardes@ccs-labs.org>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see http://www.gnu.org/licenses/.
//

#ifndef HUMANINTERFERINGPROTOCOL_H_
#define HUMANINTERFERINGPROTOCOL_H_

#include "veins/base/modules/BaseApplLayer.h"
#include "veins/modules/application/platooning/protocols/BaseProtocol.h"

#include "veins/modules/application/platooning/UnicastProtocol.h"
#include "veins/modules/application/platooning/messages/PlatooningBeacon_m.h"

#include "veins/modules/mobility/traci/TraCIMobility.h"

#include "veins/modules/application/platooning/utilities/BasePositionHelper.h"

#include "veins/modules/mac/ieee80211p/Mac1609_4.h"

class HumanInterferingProtocol : public BaseApplLayer {

	private:
        bool laneChanged;
		//beacon interval
		SimTime beaconingInterval;
		//access category
		int priority;
		//packet size
		int packetSize;
		//transmit power in mW
		double txPower;
		//bit rate in bps
		int bitrate;
		// too release the fixed lane
		cMessage* releaseFixedLane = new cMessage();
		long id;
		long currentLane = 0;

		//Number of lanes is fixed in this scenario
		int numLanes = 3;
	protected:

		//traci mobility. used for getting/setting info about the car
		Veins::TraCIMobility *mobility;
		Veins::TraCICommandInterface *traci;
		Veins::TraCICommandInterface::Vehicle *traciVehicle;

		//pointer to the mac layer
		Mac1609_4 *mac;

		//messages for scheduleAt
		cMessage *sendBeacon;

		virtual void handleSelfMsg(cMessage *msg);

        virtual void handleLowerMsg(cMessage *msg);
        virtual void handleMsg(cMessage *msg);


        /**
         * Sends a BSM packet
         */
		void sendBSM();
	public:

		//id for beacon message
        static const int INTERFERENCE_TYPE = 12349;
        static const int MANEUVER_TYPE = 1337;

		HumanInterferingProtocol() {
			sendBeacon = nullptr;
		}
		virtual ~HumanInterferingProtocol();

		virtual void initialize(int stage);

};

#endif
