//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see http://www.gnu.org/licenses/.
// 

#include "PlatoonsPlusHumanAndRandomTraffic.h"

Define_Module(PlatoonsPlusHumanAndRandomTraffic);


void PlatoonsPlusHumanAndRandomTraffic::initialize(int stage) {

    TraCIBaseTrafficManager::initialize(stage);

    if (stage == 0) {

        nCars = par("nCars").longValue();
        platoonSize = par("platoonSize").longValue();
        nLanes = par("nLanes").longValue();
        humanCars = par("humanCars").longValue();
        humanLanes = par("humanLanes").longValue();
        platoonInsertTime = SimTime(par("platoonInsertTime").doubleValue());
        platoonInsertSpeed = par("platoonInsertSpeed").doubleValue();
        platoonInsertDistance = par("platoonInsertDistance").doubleValue();
        platoonInsertHeadway = par("platoonInsertHeadway").doubleValue();
        platoonLeaderHeadway = par("platoonLeaderHeadway").doubleValue();
        platooningVType = par("platooningVType").stdstringValue();
        humanVType = par("humanVType").stdstringValue();
        insertPlatoonMessage = new cMessage("");
        scheduleAt(platoonInsertTime, insertPlatoonMessage);

    }

}

void PlatoonsPlusHumanAndRandomTraffic::scenarioLoaded() {
    automated.id = findVehicleTypeIndex(platooningVType);
    automated.lane = -1;
    automated.position = 0;
    automated.speed = platoonInsertSpeed/3.6;
    human.id = findVehicleTypeIndex(humanVType);
    human.lane = -1;
    human.position = 0;
    human.speed = platoonInsertSpeed/3.6 - 0.01;

    rCar.id = findVehicleTypeIndex(humanVType);
    rCar.lane = -1;
    rCar.position = 0;
    rCar.speed = platoonInsertSpeed/3.6 - 0.01;
}

void PlatoonsPlusHumanAndRandomTraffic::handleSelfMsg(cMessage *msg) {

    TraCIBaseTrafficManager::handleSelfMsg(msg);

    if (msg == insertPlatoonMessage) {
        insertPlatoons();
       // insertHumans();
        //insertRandomTraffic();
    }

}

void PlatoonsPlusHumanAndRandomTraffic::insertRandomTraffic() {

    //search for the scenario manager. it will be needed to inject vehicles
    manager = FindModule<Veins::TraCIScenarioManager*>::findGlobalModule();
    commandInterface = manager->getCommandInterface();

    int numLanes = 4;
    double *laneOffset = new double[numLanes];
    for (int l = 0; l < numLanes; l++) {
        laneOffset[l] = uniform(0, 20);
    }
    double currentPos = 0;
    for(auto i = 0; i < 50; i++){//Use flows
        int lane = intuniform(0, numLanes);
        if(lane == 1)
            continue;
        rCar.position = currentPos + laneOffset[lane];
        rCar.lane = lane;
        //addVehicleToQueue(0, rCar);
        std::string type = "vtypehuman";
        std::string route = "platoon_route";
        /*auto id = rCar.id;
        auto suc = commandInterface->addVehicle(type, type, route, simTime(), (double)rCar.position, (double)rCar.speed,rCar.lane);*/
    }
}

void PlatoonsPlusHumanAndRandomTraffic::insertPlatoons() {

    //compute intervehicle distance
    double distance = platoonInsertSpeed / 3.6 * platoonInsertHeadway + platoonInsertDistance;
    //total number of platoons per lane
    int nPlatoons = nCars / platoonSize / nLanes;
    //length of 1 platoon
    double platoonLength = platoonSize * 4 + (platoonSize - 1) * distance;
    //inter-platoon distance
    double platoonDistance = platoonInsertSpeed / 3.6 * platoonLeaderHeadway;
    //total length for one lane
    double totalLength = nPlatoons * platoonLength + (nPlatoons - 1) * platoonDistance;

    //for each lane, we create an offset to have misaligned platoons
    double *laneOffset = new double[nLanes];
    for (int l = 0; l < nLanes; l++)
        laneOffset[l] = uniform(0, 20);

    double currentPos = totalLength;
    int currentCar = 0;
    for (int i = 0; i < nCars/nLanes; i++) {
        for (int l = 0; l < nLanes; l++) {
            automated.position = currentPos + laneOffset[l];
            automated.lane = 1;
            addVehicleToQueue(0, automated);
        }
        currentCar++;
        if (currentCar == platoonSize) {
            currentCar = 0;
            //add inter platoon gap
            currentPos -= (platoonDistance + 4);
        }
        else {
            //add intra platoon gap
            currentPos -= (4 + distance);
        }
    }

    delete [] laneOffset;

}

void PlatoonsPlusHumanAndRandomTraffic::insertHumans() {

    //keep 50 m between human vehicles (random number)
    double distance = 50;
    //total number of cars per lane
    int carsPerLane = humanCars / humanLanes;
    //total length for one lane
    double totalLength = carsPerLane * (4 + distance);

    //for each lane, we create an offset to have misaligned platoons
    double *laneOffset = new double[humanLanes];
    for (int l = 0; l < humanLanes; l++)
        laneOffset[l] = uniform(0, 20);

    double currentPos = totalLength;
    for (int i = 0; i < carsPerLane; i++) {
        for (int l = nLanes; l < humanLanes + nLanes; l++) {
            human.position = currentPos + laneOffset[l-nLanes];
            human.lane = l;
            addVehicleToQueue(0, human);
        }
        currentPos -= (4 + distance);
    }

    delete [] laneOffset;

}

PlatoonsPlusHumanAndRandomTraffic::~PlatoonsPlusHumanAndRandomTraffic() {
    cancelAndDelete(insertPlatoonMessage);
    insertPlatoonMessage = nullptr;
}
